package es.luisbelloch.kai;

import org.apache.avro.mapred.AvroKey;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import scala.Function1;

import java.io.IOException;

public class FilterMapper
        extends Mapper<AvroKey, NullWritable, AvroKey, NullWritable> {

    public final static String FILTER_FNC_PTR = "kuduFilterFunc";
    private final static Logger logger = LogManager.getLogger(FilterMapper.class);

    private Class<?> filterClass;

    @Override
    protected void setup(Context context) throws IOException, InterruptedException {
        logger.setLevel(Level.DEBUG);
        logger.debug("Initializing FilterMapper");
        Configuration cfg = context.getConfiguration();
        String filterDescriptor = cfg.get(FILTER_FNC_PTR);
        logger.debug("Filter Descriptor: " + filterDescriptor);
        filterClass = cfg.getClassByNameOrNull(filterDescriptor);

        if (filterClass == null)
            logger.error("Unable to find filter class: " + filterDescriptor);
    }

    @Override
    @SuppressWarnings("unchecked")
    protected void map(AvroKey key, NullWritable ignore, Context context)
            throws IOException, InterruptedException {
        try {
            final Object dataEntity = key.datum();
            Function1 filter = (Function1)filterClass.newInstance();
            Boolean result = (Boolean)filter.apply(dataEntity);
            if (result)
                context.write(new AvroKey(dataEntity), NullWritable.get());
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    }
}
