package es.luisbelloch.kai;

import org.apache.avro.Schema;
import org.apache.avro.mapreduce.AvroJob;
import org.apache.avro.mapreduce.AvroKeyInputFormat;
import org.apache.avro.mapreduce.AvroKeyValueOutputFormat;
import org.apache.avro.reflect.ReflectData;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.IOException;

public class FoldJobFactory implements JobFactory {

    protected Logger logger = LogManager.getLogger(getClass());

    @Override
    public Job create(FlowStep step, String input, String output, Configuration cfg) throws IOException {
        logger.setLevel(Level.DEBUG);
        logger.debug("Job Step: " + step.toString());
        logger.debug("Input extent: " + input);
        logger.debug("Output extent: " + output);

        cfg.set(FoldReducer.FOLD_FNC_PTR, step.ptr());
        cfg.set(FoldReducer.FOLD_ACC_PTR, step.getArg(0)._2());
        cfg.set(FoldReducer.FOLD_ACC_VAL, step.getArg(0)._1());

        final Job job = Job.getInstance(cfg, getClass().getName() + "/" + step.ptr());
        job.setJarByClass(FoldMapper.class);

        FileInputFormat.setInputPaths(job, new Path(input));
        FileOutputFormat.setOutputPath(job, new Path(output));

        Schema inputSchema = ReflectData.get().getSchema(step.io()._1());
        Schema stringSchema = Schema.create(Schema.Type.STRING);
        Schema outputSchema = ReflectData.get().getSchema(step.io()._2());

        job.setInputFormatClass(AvroKeyInputFormat.class);
        job.setOutputFormatClass(AvroKeyValueOutputFormat.class);

        AvroJob.setInputKeySchema(job, inputSchema);
        // AvroJob.setInputValueSchema(job, null);
        AvroJob.setOutputKeySchema(job, stringSchema);
        AvroJob.setOutputValueSchema(job, outputSchema);

        AvroJob.setMapOutputKeySchema(job, stringSchema);
        AvroJob.setMapOutputValueSchema(job, inputSchema);

        job.setMapperClass(FoldMapper.class);
        job.setReducerClass(FoldReducer.class);

        return job;
    }
}
