package es.luisbelloch.kai;

import org.apache.avro.mapred.AvroKey;
import org.apache.avro.mapred.AvroValue;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.mapreduce.Reducer;
import org.codehaus.jackson.map.ObjectMapper;
import scala.Function2;

import java.io.IOException;

public class FoldReducer
        extends Reducer<AvroKey, AvroValue, AvroKey, AvroValue> {

    public final static String FOLD_FNC_PTR = "kuduFoldFunc";
    public final static String FOLD_ACC_PTR = "kuduFoldAccClass";
    public final static String FOLD_ACC_VAL = "kuduFoldAccValue";

    public final static ObjectMapper objectMapper = new ObjectMapper();

    private Class<?> predicateClass;
    private Class<?> accumulatorClass;
    private String accumulatorValue;

    @Override
    protected void setup(Context context) throws IOException, InterruptedException {
        Configuration configuration = context.getConfiguration();
        predicateClass = Reflector.getClassFromDescriptor(FOLD_FNC_PTR, configuration);
        accumulatorClass = Reflector.getClassFromDescriptor(FOLD_ACC_PTR, configuration);
        accumulatorValue = configuration.get(FOLD_ACC_VAL);
    }

    @Override
    protected void reduce(AvroKey key, Iterable<AvroValue> values, Context context)
            throws IOException, InterruptedException {

        final Function2 predicate = Reflector.getPredicateFromClass(predicateClass);
        Object acc = objectMapper.readValue(accumulatorValue, accumulatorClass);

        for (AvroValue value: values) {
            acc = predicate.apply(acc, value.datum());
        }

        context.write(new AvroKey(""), new AvroValue(acc));
    }
}
