package es.luisbelloch.kai;

import org.apache.avro.Schema;
import org.apache.avro.mapreduce.AvroJob;
import org.apache.avro.mapreduce.AvroKeyInputFormat;
import org.apache.avro.mapreduce.AvroKeyOutputFormat;
import org.apache.avro.reflect.ReflectData;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.IOException;

public abstract class MapOnlyJobFactory implements JobFactory {

    protected Logger logger = LogManager.getLogger(getClass());

    private final String descriptorName;
    private final Class<? extends Mapper> mapper;

    protected MapOnlyJobFactory(String descriptorName, Class<? extends Mapper> mapper) {
        logger.setLevel(Level.DEBUG);
        this.descriptorName = descriptorName;
        this.mapper = mapper;
    }

    @Override
    public Job create(FlowStep step, String input, String output, Configuration cfg) throws IOException {
        logger.debug("Job Step: " + step.toString());
        logger.debug("Input extent: " + input);
        logger.debug("Output extent: " + output);

        cfg.set(descriptorName, step.ptr());

        final Job job = Job.getInstance(cfg, getClass().getName() + "/" + step.ptr());
        job.setJarByClass(mapper);

        FileInputFormat.setInputPaths(job, new Path(input));
        FileOutputFormat.setOutputPath(job, new Path(output));

        job.setInputFormatClass(AvroKeyInputFormat.class);
        job.setOutputFormatClass(AvroKeyOutputFormat.class);

        Schema inputSchema = ReflectData.get().getSchema(step.io()._1());
        Schema outputSchema = ReflectData.get().getSchema(step.io()._2());

        AvroJob.setInputKeySchema(job, inputSchema);
        AvroJob.setOutputKeySchema(job, outputSchema);
        logger.debug("Input schema: " + inputSchema.toString());
        logger.debug("Output schema: " + outputSchema.toString());

        AvroJob.setMapOutputKeySchema(job, outputSchema);
        job.setMapOutputValueClass(NullWritable.class);

        job.setMapperClass(mapper);
        job.setNumReduceTasks(0);

        return job;
    }
}
