package es.luisbelloch.kai;

import org.apache.hadoop.conf.Configuration;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

public class Reflector {

    private final static Logger logger = LogManager.getLogger(Reflector.class);

    static {
        LogManager.getRootLogger().setLevel(Level.DEBUG);
    }

    public static Class<?> getClassFromDescriptor(String key, Configuration cfg) {
        final String descriptor = cfg.get(key);
        final Class<?> predicate = cfg.getClassByNameOrNull(descriptor);
        if (predicate == null)
            logger.error("Unable to find predicate class: " + descriptor);

        logger.debug("getClassFromDescriptor(" + key + "): " + descriptor);
        return predicate;
    }

    @SuppressWarnings("unchecked")
    public static <T> T getPredicateFromClass(Class<?> classReference) {
        try {
        final Object predicate = classReference.newInstance();
        if (classReference.isAssignableFrom(predicate.getClass()))
            return (T) predicate;
        } catch (InstantiationException e) {
            logger.fatal("Unable to instantiate class: " + classReference.getName(), e);
        } catch (IllegalAccessException e) {
            logger.fatal("Illegal access to class constructor: " + classReference.getName(), e);
        }
        return null;
    }
}
