package es.luisbelloch.kai;

import org.apache.avro.mapred.AvroKey;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import scala.Function1;

import java.io.IOException;

public class TransformMapper
        extends Mapper<AvroKey, NullWritable, AvroKey, NullWritable> {

    public final static String MAP_FNC_PTR = "kuduTransformFunc";
    private final static Logger logger = LogManager.getLogger(FilterMapper.class);

    private Class<?> transformClass;

    @Override
    protected void setup(Mapper.Context context) throws IOException, InterruptedException {
        logger.setLevel(Level.DEBUG);
        logger.debug("Initializing FilterMapper");
        Configuration cfg = context.getConfiguration();
        String transformDescriptor = cfg.get(MAP_FNC_PTR);
        logger.debug("Transform descriptor: " + transformDescriptor);
        transformClass = cfg.getClassByNameOrNull(transformDescriptor);

        if (transformClass == null)
            logger.error("Unable to find transform class: " + transformDescriptor);
    }

    @Override
    @SuppressWarnings("unchecked")
    protected void map(AvroKey key, NullWritable ignore, Context context)
            throws IOException, InterruptedException {

        try {
            final Object dataEntity = key.datum();
            Function1 transform = (Function1)transformClass.newInstance();
            Object result = transform.apply(dataEntity);
            context.write(new AvroKey(result), NullWritable.get());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
