package playground;

import org.apache.avro.Schema;
import org.apache.avro.io.*;
import org.apache.avro.reflect.ReflectData;
import org.apache.avro.reflect.ReflectDatumReader;
import org.apache.avro.reflect.ReflectDatumWriter;
import playground.demos.Persona;
import playground.demos.PlaygroundFactory;
import playground.demos.ShareClass;
import scala.Function1;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Arrays;

public class AvroSample {

    public static void testSchemaUnions() throws IOException {
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        Encoder encoder = EncoderFactory.get().binaryEncoder(os, null);

        final Schema schPersona = ReflectData.get().getSchema(Persona.class);
        final Schema schShareClass = ReflectData.get().getSchema(ShareClass.class);
        final Schema union = Schema.createUnion(Arrays.asList(schPersona, schShareClass));

        System.out.println(union);

        DatumWriter writer = new ReflectDatumWriter(union);
//        writer.write(new ShareClass("Bond", "USD", 12), encoder);
        writer.write(null, encoder);
        writer.write(new Persona("Jack Sparrow", 27), encoder);
        writer.write(new Persona("Mike Wazowski", 30), encoder);
        writer.write(new Persona("Polanski", 60), encoder);
        encoder.flush();

        System.out.println(os.toString());

        Decoder decoder = DecoderFactory.get().binaryDecoder(os.toByteArray(), null);
        //  DatumReader reader = new ReflectDatumReader(ShareClass.class);

        DatumReader reader = new ReflectDatumReader(union);
//        SpecificDatumReader reader1 = new SpecificDatumReader(union);

        System.out.println(reader.read(null, decoder)); // read: playground.Persona@7b36a43c
        System.out.println(reader.read(null, decoder));
        System.out.println(reader.read(null, decoder));
        System.out.println(reader.read(null, decoder));
    }

    public static void testNormalSerialization() throws IOException {
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        Encoder encoder = EncoderFactory.get().binaryEncoder(os, null);

        ShareClass shareClass = new ShareClass("Bond", "USD", 12);
        DatumWriter writer = new ReflectDatumWriter(shareClass.getClass());
        writer.write(shareClass, encoder);
        encoder.flush();

        System.out.println(os.toString());

        Decoder decoder = DecoderFactory.get().binaryDecoder(os.toByteArray(), null);
        DatumReader reader = new ReflectDatumReader(shareClass.getClass());
        Object hydrated = reader.read(null, decoder);
        System.out.println(hydrated);
    }

    public static void testSomeTransformation() throws Exception {
        String ptrToClass = PlaygroundFactory.firstMapperOps();
        Class<?> objClass = Class.forName(ptrToClass);
        Function1 obj = (Function1)objClass.newInstance();
        Object transformed = obj.apply(new ShareClass("Bond", "USD", 12));
        System.out.println(transformed.getClass().getName());
    }

    public static void main(String[] args) throws Exception {
        testSchemaUnions();
    }
}
