package playground;

import com.google.common.collect.Iterables;
import es.luisbelloch.kai.DataFlow;
import es.luisbelloch.kai.FilterFunc;
import es.luisbelloch.kai.Mutator;
import playground.demos.PlaygroundFactory;
import playground.demos.ShareClass;
import scala.Function1;

import java.lang.reflect.Constructor;
import java.util.Collection;

public class ReflectionSample {

    public static void main(String[] args) throws Exception {
        String className = "es.luisbelloch.kai.FoldFunc$$anonfun$1";
        final Class<?> clazz = Class.forName(className);
        final Constructor<?>[] constructors = clazz.getConstructors();
        for (Constructor<?> constructor : constructors) {
            final Class<?>[] parameterTypes = constructor.getParameterTypes();
            for (Class<?> parameterType : parameterTypes) {
                System.out.println(parameterType.getName());
            }
        }

//        String name1 = getFirstOperationPredicateName(PlaygroundFactory.someFilter());
//        String name2 = getFirstOperationPredicateName(PlaygroundFactory.otherFilter());
//        String name3 = getFirstOperationThroughReflect(PlaygroundFactory.someFilter());
//        createOperation(name3);
//
//        String name4 = PlaygroundFactory.firstOperationUsingReflect();
//        System.out.println("Predicate: " + name4);
//        createOperation(name4);
    }

    private static String getFirstOperationPredicateName(DataFlow<ShareClass> dataFlow) {
        Collection<Mutator> operations = PlaygroundFactory.operations(dataFlow);
        Mutator mutator = Iterables.getFirst(operations, null);
        FilterFunc<ShareClass> filter = (FilterFunc<ShareClass>) mutator;
        String predicateName = filter.predicate().getClass().getName();
        System.out.println("Predicate: " + predicateName);
        System.out.println("Superclass: " + filter.predicate().getClass().getSuperclass());
        System.out.println("Filter: " + filter.getClass().getName());
        return predicateName;
    }

    private static String getFirstOperationThroughReflect(DataFlow<ShareClass> dataFlow) {
        Collection<Mutator> operations = PlaygroundFactory.operations(dataFlow);
        Mutator mutator = Iterables.getFirst(operations, null);
        FilterFunc<ShareClass> filter = (FilterFunc<ShareClass>) mutator;
        String predicateName = filter.reflect();
        System.out.println("Predicate: " + predicateName);
        return predicateName;
    }

    private static void createOperation(String predicateClassName) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
        Class<?> filterClass = Class.forName(predicateClassName);
        Function1 filter = (Function1)filterClass.newInstance();
        ShareClass shareClass = new ShareClass("A", "US", 13);
        System.out.println(filter.apply(shareClass));
    }
}
