package playground.samples.Avro;

import org.apache.avro.Schema;
import org.apache.avro.mapred.AvroKey;
import org.apache.avro.mapreduce.AvroJob;
import org.apache.avro.mapreduce.AvroKeyInputFormat;
import org.apache.avro.mapreduce.AvroKeyOutputFormat;
import org.apache.avro.reflect.ReflectData;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.util.Tool;
import org.apache.hadoop.util.ToolRunner;
import playground.demos.ShareClass;

import java.io.IOException;

public class AvroFilterApplication extends Configured implements Tool {

    public static class AvroFilterMapper
        extends Mapper<AvroKey<ShareClass>, NullWritable, AvroKey<ShareClass>, NullWritable> {

        @Override
        protected void map(AvroKey<ShareClass> shareClassAvroKey, NullWritable ignored, Context context)
                throws IOException, InterruptedException {

            final ShareClass shareClass = shareClassAvroKey.datum();
            if (shareClass.currency().equals("USD"))
                context.write(new AvroKey<ShareClass>(shareClass), NullWritable.get());
                // context.write(shareClassAvroKey, NullWritable.get());
        }
    }

    @Override
    public int run(String[] args) throws Exception {
        if (args.length != 2) {
            System.err.printf("Usage: %s [generic options] <input> <output>\n", getClass().getSimpleName());
            ToolRunner.printGenericCommandUsage(System.err);
            return -1;
        }

        Job job = Job.getInstance(getConf());
        job.setJobName(getClass().getName());
        job.setJarByClass(getClass());

        Schema schema = ReflectData.get().getSchema(ShareClass.class);

        FileInputFormat.setInputPaths(job, new Path(args[0]));
        FileOutputFormat.setOutputPath(job, new Path(args[1]));

        job.setInputFormatClass(AvroKeyInputFormat.class);
        job.setOutputFormatClass(AvroKeyOutputFormat.class);

        AvroJob.setInputKeySchema(job, schema);
        AvroJob.setOutputKeySchema(job, schema);

        AvroJob.setMapOutputKeySchema(job, schema);
        job.setMapOutputValueClass(NullWritable.class);

        job.setMapperClass(AvroFilterMapper.class);
        job.setNumReduceTasks(0);

        return job.waitForCompletion(true) ? 0 : 1;
    }

    public static void main(String[] args) throws Exception {
        Configuration conf = new Configuration();
        int exitCode = ToolRunner.run(conf, new AvroFilterApplication(), args);
        System.exit(exitCode);
    }
}
