package playground.samples.Avro;

import org.apache.avro.Schema;
import org.apache.avro.mapred.AvroKey;
import org.apache.avro.mapred.AvroValue;
import org.apache.avro.mapreduce.*;
import org.apache.avro.reflect.ReflectData;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.util.Tool;
import org.apache.hadoop.util.ToolRunner;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import playground.demos.ShareClass;
import scala.Function1;

import java.io.IOException;

public class AvroFoldApplication extends Configured implements Tool {

    private final static Logger logger = LogManager.getLogger(AvroFoldApplication.class);

    public static class AvroFoldMapper
            extends Mapper<AvroKey<ShareClass>, NullWritable, AvroKey, AvroValue<ShareClass>> {

        private Function1 groupFnc;

        @Override
        protected void setup(Context context) throws IOException, InterruptedException {
            try {
                Configuration cfg = context.getConfiguration();
                Class<?> groupFncClass = cfg.getClassByNameOrNull(
                    "playground.ScalaPlayground$$anonfun$shareClassGroupKeyExtractorLambda$1");
                groupFnc = (Function1)groupFncClass.newInstance();
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }

        @Override
        protected void map(AvroKey<ShareClass> shareClassAvroKey, NullWritable ignored, Context context)
                throws IOException, InterruptedException {
            final ShareClass shareClass = shareClassAvroKey.datum();

            // Grouped by currency
            // context.write(new AvroKey(groupFnc.apply(shareClass)), new AvroValue<ShareClass>(shareClass));

            // Complete sum of all share classes
            context.write(new AvroKey(""), new AvroValue<ShareClass>(shareClass));
        }
    }

    public static class AvroFoldReducer
            extends Reducer<AvroKey, AvroValue<ShareClass>, AvroKey, AvroValue<Double>> {

        @Override
        protected void reduce(AvroKey key, Iterable<AvroValue<ShareClass>> values, Context context)
                throws IOException, InterruptedException {

            Double total = 0d; // B
            for (AvroValue<ShareClass> value : values)
                total += value.datum().price(); // predicate

            // context.write(new AvroKey(key.toString()), new AvroValue<Double>(total));
            context.write(new AvroKey(""), new AvroValue<Double>(total));
        }
    }

    @Override
    public int run(String[] args) throws Exception {
        if (args.length != 2) {
            System.err.printf("Usage: %s [generic options] <input> <output>\n", getClass().getSimpleName());
            ToolRunner.printGenericCommandUsage(System.err);
            return -1;
        }

        Job job = Job.getInstance(getConf());
        job.setJobName(getClass().getName());
        job.setJarByClass(getClass());

        FileInputFormat.setInputPaths(job, new Path(args[0]));
        FileOutputFormat.setOutputPath(job, new Path(args[1]));

//        Function1<ShareClass, String> extractor = ScalaPlayground.shareClassGroupKeyExtractorLambda();
//        String groupPtr = extractor.getClass().getName();
//        logger.setLevel(Level.DEBUG);
//        logger.debug("groupPtr: " + groupPtr);
//        getConf().set(AvroFoldMapper.GROUP_FNC_PTR, groupPtr);

        Schema shareClassSchema = ReflectData.get().getSchema(ShareClass.class);
        Schema doubleSchema = Schema.create(Schema.Type.DOUBLE);
        Schema stringSchema = Schema.create(Schema.Type.STRING);

        job.setInputFormatClass(AvroKeyInputFormat.class);
        job.setOutputFormatClass(AvroKeyValueOutputFormat.class);

        AvroJob.setInputKeySchema(job, shareClassSchema);
        // AvroJob.setInputValueSchema(job, none here!);
        AvroJob.setOutputKeySchema(job, stringSchema);
        AvroJob.setOutputValueSchema(job, doubleSchema); // Cambio si group, should be list?

        AvroJob.setMapOutputKeySchema(job, stringSchema); // Debería ser del tipo de retorno de la lambda
        AvroJob.setMapOutputValueSchema(job, shareClassSchema); // Cambio si group, shoudl be list?


        job.setMapperClass(AvroFoldMapper.class);
        job.setReducerClass(AvroFoldReducer.class);
        // job.setNumReduceTasks(1);

        return job.waitForCompletion(true) ? 0 : 1;
    }

    public static void main(String[] args) throws Exception {
        Configuration conf = new Configuration();
        int exitCode = ToolRunner.run(conf, new AvroFoldApplication(), args);
        System.exit(exitCode);
    }
}
