package es.luisbelloch.kai

import org.apache.hadoop.conf.{Configuration, Configured}
import org.apache.hadoop.util.{ToolRunner, Tool}
import java.net.URI

abstract class BasicApplication extends Configured with Tool {
  val name: String
  val flow: DataFlow[_]

  override def run(args: Array[String]): Int = {
    if (args.length != 1) {
      System.err.printf("Usage: %s [generic options] <workingDir>\n", name);
      ToolRunner.printGenericCommandUsage(System.err);
      -1
    }

    val base = new URI(args(0))
    val cfg = new FlowConfig(name, base, getConf)
    val result = Kudu.submit(flow, cfg)

    if (result.succeeded) 0 else -1
  }

  def main(args: Array[String]) {
    val cfg = new Configuration()
    val exitCode = ToolRunner.run(cfg, this, args)
    System.exit(exitCode)
  }
}