package es.luisbelloch.kai

trait Controller {
  def submit(df: DataFlow[_], cfg: FlowConfig): FlowResult
}

class DefaultJobController(scheduler: Scheduler) extends Controller {
  def submit(df: DataFlow[_], cfg: FlowConfig): FlowResult = {
    val plan = scheduler.plan(df)
    val results = plan map { e => submitOne(e, cfg, plan.length) }
    new FlowResult(succeeded = results.last)
  }

  private def submitOne(step: FlowStep, cfg: FlowConfig, expectedJobs: Int) = {
    var input = cfg.input
    if (step.junction._1 != 0)
      input = cfg.tmp + "/t" + step.junction._1

    var output = cfg.tmp + "/t" + step.junction._2
    if (step.junction._2 == expectedJobs)
      output = cfg.output

    val job = HadoopJobFactory.create(step, input, output, cfg.hadoopCfg)
    job.waitForCompletion(true)
  }
}

object Kudu extends Controller {
  var scheduler: Scheduler = new BasicScheduler
  var controller: Controller = new DefaultJobController(scheduler)
  def submit(df: DataFlow[_], cfg: FlowConfig) = controller.submit(df, cfg)
}
