package es.luisbelloch.kai

import org.apache.hadoop.conf.Configuration
import org.apache.hadoop.mapreduce.{Job => HJob}
import org.apache.log4j.LogManager

object HadoopJobFactory {
  val log = LogManager.getLogger(getClass.getName)
  val filterJobFactory: JobFactory = new FilterJobFactory
  val transformJobFactory: JobFactory = new TransformJobFactory
  val foldJobFactory: JobFactory = new FoldJobFactory

  def create(step: FlowStep, input: String, output: String, cfg: Configuration): HJob = {
    step.op match {
      case filter: FilterFunc[_] => filterJobFactory.create(step, input, output, cfg)
      case map: TransformFunc[_,_] => transformJobFactory.create(step, input, output, cfg)
      case fold: FoldFunc[_,_] => foldJobFactory.create(step, input, output, cfg)
      case _ => throw new UnsupportedOperationException()
    }
  }
}
