package es.luisbelloch.kai

trait Optimizer {
  def solve(original: List[FlowStep]): List[FlowStep]
}

object Optimizer {
  def compress[T](l : List[T], eq: (T,T) => Boolean, merge: (T,T) => T) : List[T] = l match {
    case head::next::tail if eq(head,next) => compress(merge(head,next)::tail, eq, merge)
    case head::tail => head::compress(tail, eq, merge)
    case nil => List()
  }
}

object MergeConsecutive {
  import Optimizer._

  def canMerge(a: FlowStep, b: FlowStep): Boolean =
    (a.op.isInstanceOf[FilterFunc[_]] || a.op.isInstanceOf[TransformFunc[_,_]]) &&
     a.op.getClass.equals(b.op.getClass)

  def merge[T,Q,R](a: FlowStep, b: FlowStep): FlowStep = (a.op, b.op) match {
    case (p: FilterFunc[T], q: FilterFunc[T]) => {
      val comp: (T) => Boolean = t => p.predicate.apply(t) && q.predicate.apply(t)
      val func = new FilterFunc[T](comp)
      new FlowStep(func, func.reflect(), (a.junction._1, b.junction._2), a.io)
    }
    case (p: TransformFunc[T,Q], q: TransformFunc[Q,R]) => {
      val comp = q.predicate compose p.predicate
      val func = new TransformFunc[T,R](comp)
      new FlowStep(func, func.reflect(), (a.junction._1, b.junction._2), (a.io._1, b.io._2))
    }
  }

  def apply(): Optimizer = new Optimizer {
    def solve(original: List[FlowStep]): List[FlowStep] =
      compress(original, canMerge, merge)
  }
}

