package es.luisbelloch.kai

class DataFlow[T]() {
  var operations: List[Mutator] = List()
  var typeSequence: List[(Class[_], Class[_])] = List()

  def this(flow: DataFlow[_], mutator: Mutator, inType: Class[_], outType: Class[_]) = {
    this()
    operations = operations ++ (flow.operations :+ mutator)
    typeSequence = typeSequence ++ (flow.typeSequence :+ (inType, outType))
  }
}

object DataFlow {
  def apply[T] = new DataFlow[T]()
}

trait Mutator {
  def reflect(): String
}

class FilterFunc[T](val predicate: T => Boolean) extends Mutator {
  def reflect(): String = predicate.getClass.getName
}

object Filter {
  def apply[T](flow: DataFlow[T], predicate: (T => Boolean), tag: Class[_]): DataFlow[T] = {
    new DataFlow[T](flow, new FilterFunc(predicate), tag, tag)
  }
}

class TransformFunc[T,K](val predicate: T => K) extends Mutator {
  def reflect(): String = predicate.getClass.getName
}

object Transform {
  def apply[T,K](flow: DataFlow[T], predicate: T => K, inType: Class[_], outType: Class[_]): DataFlow[K] = {
    new DataFlow[K](flow, new TransformFunc(predicate), inType, outType)
  }
}

class FoldFunc[B,A](val predicate: (B, A) => B, val init: B) extends Mutator {
  val reflectInitial = init.getClass.getName
  def reflect(): String = predicate.getClass.getName
}

object Fold {
  def apply[B,A](flow: DataFlow[A], predicate: (B, A) => B, init: B, in: Class[_], out: Class[_]): DataFlow[B] = {
    new DataFlow[B](flow, new FoldFunc(predicate, init), in, out)
  }
}
