package es.luisbelloch.kai

import org.codehaus.jackson.map.ObjectMapper

trait Scheduler {
  def plan(flow: DataFlow[_]): List[FlowStep]
}

class BasicScheduler extends Scheduler {

  private val json: ObjectMapper = new ObjectMapper

  def inferArgs(m: Mutator): List[(String,String)] = {
    m match {
      case f: FoldFunc[_,_] => List((json.writeValueAsString(f.init), f.reflectInitial))
      case _ => List()
    }
  }

  def plan(flow: DataFlow[_]): List[FlowStep] = {
    val ops = flow.operations zip flow.typeSequence map {
      f => (f._1, f._1.reflect(), inferArgs(f._1), f._2)
    }
    val jnt = ops.foldLeft(List(Pair(0, 1)))((B, A) => B :+ Pair(B.last._2, B.last._2 + 1))
    jnt zip ops map { e => new FlowStep(e._2._1, e._2._2, e._1, e._2._4, e._2._3) }
  }
}

class OptimizedScheduler(optimizers: Optimizer*)
  extends BasicScheduler {
  override def plan(flow: DataFlow[_]): List[FlowStep] = {
    val original = super.plan(flow)
    optimizers.foldLeft(original)((f,opt) => opt.solve(f)).toList
  }
}
