package es.luisbelloch.kai

import scala.reflect.ClassTag

object Support {

  def size[T](flow: DataFlow[T])(implicit inTag: ClassTag[T]): DataFlow[Int] = {
    Fold[Int,T](flow, (a,b) => a + 1, 0, inTag.runtimeClass, classOf[Int])
  }

  def equals   [T <: AnyVal](flow: DataFlow[T], value: T) = areEquals(flow, (_:T).==(_:T), value)
  def notEquals[T <: AnyVal](flow: DataFlow[T], value: T) = areEquals(flow, (_:T).!=(_:T), value)

  def areEquals[T <: AnyVal](flow: DataFlow[T], test: (T,T) => Boolean, value: T): DataFlow[Boolean] = {
    Transform[T, Boolean](flow, k => test(k, value), classOf[AnyVal], classOf[Boolean])
  }

  def sum[T : Numeric](flow: DataFlow[T])(implicit inTag: ClassTag[T]): DataFlow[T] = {
    import scala.math.Numeric.Implicits._
    numericReduce(flow, (a,b) => a + b, inTag.runtimeClass)
  }

  def numericReduce[T : Numeric](flow: DataFlow[T], predicate: (T,T) => T, clazz: Class[_]): DataFlow[T] = {
    Fold[T,T](flow, predicate, implicitly[Numeric[T]].zero, clazz, clazz)
  }
}