package es.luisbelloch

import scala.reflect.ClassTag

package object kai {

  import es.luisbelloch.kai.Support._
  import es.luisbelloch.kai.FlatLand._

  implicit def op[T,Q](flow: DataFlow[T])(implicit inTag: ClassTag[T]) = new {

    def filter(predicate: T => Boolean) = Filter(flow, predicate, inTag.runtimeClass)

    def transform[K](predicate: T => K)(implicit outTag: ClassTag[K]): DataFlow[K] = {
      Transform[T,K](flow, predicate, inTag.runtimeClass, outTag.runtimeClass)
    }

    def fold[B >: Q](init: B)(predicate: (B, T) => B)(implicit outTag: ClassTag[B]): DataFlow[B] = {
      Fold[B,T](flow, predicate, init, inTag.runtimeClass, outTag.runtimeClass)
    }

    def identity() =
      Transform[T,T](flow, t => t, inTag.runtimeClass, inTag.runtimeClass)

    def sortBy[G](keyExtractor: T => G): DataFlow[T] = identity()
    def reverse() = identity()
    def shuffle() = identity()

    def distinct()(implicit outTag: ClassTag[List[T]]) = {
      val p = (a:List[T], e:T) => {
        if (a.head == e) a
        else a :+ e
      }
      Fold(flow, p, List(), inTag.runtimeClass, outTag.runtimeClass)
    }

    def any(predicate: T => Boolean) = {
      notEquals(size(Filter(flow, predicate, inTag.runtimeClass)), 0)
    }

    def all(predicate: T => Boolean) = {
      equals(size(Filter(flow, (p: T) => !predicate(p), inTag.runtimeClass)), 0)
    }

    def count(): DataFlow[Int] = size(flow)

    def contains[E >: T](a: E) = {
      notEquals(size(any(e => e == a)), 0)
    }

    def skip(i: Int)(implicit outTag: ClassTag[List[T]]): DataFlow[T] = {
      slice(i, (a,b) => a > b)
    }

    def take(i: Int)(implicit outTag: ClassTag[List[T]]): DataFlow[T] = {
      slice(i, (a,b) => a < b)
    }

    def slice(i: Int, comparator: (Int,Int) => Boolean)(implicit outTag: ClassTag[List[T]]): DataFlow[T] = {
      def f1: ((Int,List[T]),T) => (Int,List[T]) = (p,l) => {
        if (comparator(p._1, i)) (p._1 + 1, p._2 :+ l)
        else p
      }
      val folded = Fold[(Int,List[T]),T](flow, f1, (i, List()), inTag.runtimeClass, outTag.runtimeClass)
      val list = Transform(folded, (e:(Int, List[T])) => e._2, outTag.runtimeClass, inTag.runtimeClass)
      list.flatten()
    }

    def groupBy[G](selector: T => G)(implicit outTag: ClassTag[(G, List[T])]): DataFlow[(G, List[T])] = {
      new DataFlow()
    }

    def range(a: Int, b: Int)(implicit f: DataFlow[Int] = new DataFlow[Int]): DataFlow[Int] = {
      val t = Transform(f, (t: Int) => a.to(b).toList, classOf[java.lang.Integer], classOf[java.lang.Integer])
      t.flatten()
    }

    def repeat(times: Int): DataFlow[T] = {
      val t = Transform(flow, (t: T) => { 1.to(times).map( _ => t).toList },inTag.runtimeClass, inTag.runtimeClass)
      t.flatten()
    }

    def zip[Z](other: DataFlow[Z]): DataFlow[Q] = {
      new DataFlow[Q]
      // Transform with arguments
    }

    def join[Z](other: DataFlow[Z]): DataFlow[Q] = {
      new DataFlow[Q]
      // Transform with arguments
    }
  }
}
