package playground.casos

import scala.reflect.ClassTag
import es.luisbelloch.kai._

trait Entidad {
  var id: Int = _
  var esValida: Boolean = _
}

trait Regla[T <: Entidad] {
  def marcar(entidad: T): T = {
    entidad.esValida = validar(entidad)
    entidad
  }
  def validar(entidad: T): Boolean
}

class FactoriaDeReglas[T <: Entidad] {
  def crearTrabajo(reglas: List[Regla[T]])(implicit tag: ClassTag[T]) : DataFlow[T] = {
    reglas.foldLeft(DataFlow[T])((flow, r) =>
      Transform(flow, (e:T) => r.marcar(e), tag.runtimeClass, tag.runtimeClass))
  }
}

class PagoElectronico(val importe: Double, val descuento: Double)
  extends Entidad

object EvitarDescuentosNegativos extends Regla[PagoElectronico] {
  def validar(entidad: PagoElectronico): Boolean = entidad.descuento >= 0
}

object BizApp extends BasicApplication {
  val name = "BizApp"
  val factoria = new FactoriaDeReglas[PagoElectronico]
  val flow = factoria.crearTrabajo(List(EvitarDescuentosNegativos))
}